@@ -11,11 +11,17 @@ asyncio.gather), it is required to set up a custom task factory in the event
 loop. This task factory will attach a shallow copy of the current context to
 every new task.
 """
-
+import sys
 from asyncio import Task
 from functools import wraps
 from logging import getLogger
 
+if sys.version_info >= (3, 7):
+    from asyncio import current_task
+else:
+    current_task = Task.current_task
+
+
 LOGGER = getLogger(__name__)
 
 
@@ -31,20 +37,24 @@ def with_current_task_context(func):
     """
     @wraps(func)
     def wrapper(*args, **kwargs):
-        task = Task.current_task()
+        try:
+            task = current_task()
+        except RuntimeError:
+            LOGGER.debug("Couldn't get the current task", exc_info=True)
+            task = None
         if task is None:
             # TODO This log message is displayed at startup because we set a
             # request_recorder at initialization, before the event loop is
             # started. Let's dismiss it for now.
             LOGGER.info("No current task, is the event loop running?")
-            return
+            return None
         context = getattr(task, _TASK_CONTEXT_ATTR, None)
         if context is None:
             LOGGER.error(
                 "No context attached to the current task, "
                 "is the task factory correctly set?"
             )
-            return
+            return None
         return func(context, *args, **kwargs)
 
     return wrapper
