@@ -12,21 +12,33 @@ import sys
 from os import getenv
 from os.path import expanduser, isfile
 
-from .constants import BACKEND_URL
+from .constants import BACKEND_URL, INGESTION_BACKEND_URL, LEGACY_BACKEND_URL
+from .utils import HAS_TYPING
 
 if sys.version_info[0] < 3:
     from ConfigParser import RawConfigParser, Error as ConfigError
 else:
     from configparser import RawConfigParser, Error as ConfigError
 
+if HAS_TYPING:
+    from typing import Optional, Any, Mapping
+
 LOGGER = logging.getLogger(__name__)
 
 HOME_FILE_PATH = expanduser("~/.sqreen.ini")
 
 
 CONFIG_DEFAULT_VALUE = {
+    # Agent is disabled when true
+    "DISABLE": False,
     # The backend URL
     "URL": BACKEND_URL,
+    # The Ingestion backend URL
+    "INGESTION_URL": INGESTION_BACKEND_URL,
+    # Do not attempt to detect a proxy filtering
+    "NO_SNIFF_DOMAINS": False,
+    # Legacy backend URL
+    "LEGACY_URL": LEGACY_BACKEND_URL,
     # Default token to use
     "TOKEN": None,
     # If the token is an org token, default app_name
@@ -55,7 +67,13 @@ CONFIG_DEFAULT_VALUE = {
     # Sensitive keys to strip
     "STRIP_SENSITIVE_KEYS": "password, secret, passwd, authorization, api_key, apikey, access_token",
     # Sensitive regexp (matches items like credit cards)
-    "STRIP_SENSITIVE_REGEX": r"^(?:\d[ -]*?){13,16}$"
+    "STRIP_SENSITIVE_REGEX": r"^(?:\d[ -]*?){13,16}$",
+    # Do not block longer than this period in seconds during startup
+    "INSTRUMENTATION_TIMEOUT": 6.0,
+    # Graceful timeout, max time waiting for the runner thread to exit before exiting
+    "GRACEFUL_TIMEOUT": 5.0,
+    # Static instrumentation, enabled by default
+    "BETA_STATIC_INSTRUMENTATION": True,
 }
 
 
@@ -76,7 +94,8 @@ class Config(object):
     HOME_FILE_PATH = HOME_FILE_PATH
 
     def __init__(self, default_values=None):
-        self.config = {}
+        # type: (Optional[Mapping[str, Any]]) -> None
+        self.config = {}  # type: (Mapping[str, Any])
 
         if default_values is None:
             default_values = CONFIG_DEFAULT_VALUE
@@ -131,27 +150,42 @@ class Config(object):
 
             config_dict = {}
             for option in config.options("sqreen"):
-                config_dict[option.upper()] = config.get("sqreen", option)
+                upper_option = option.upper()
+                config_dict[upper_option] = self._coerce_value(
+                    upper_option, config.get("sqreen", option))
 
             return config_dict
         except ConfigError:
             LOGGER.debug("Error parsing config file %s", file_path)
             return {}
 
-    @staticmethod
-    def load_from_env():
+    def load_from_env(self):
         """ Load configuration from os environment variables, variables
         must be prefixed with SQREEN_ to be detected.
         """
         env_config = {}
-        for env_var in os.environ:
+        for env_var, value in os.environ.items():
             if env_var.startswith("SQREEN_"):
-                env_config[env_var.replace("SQREEN_", "", 1)] = os.environ[
-                    env_var
-                ]
-
+                key = env_var[7:].upper()
+                env_config[key] = self._coerce_value(key, value)
         return env_config
 
+    def _coerce_value(self, name, value):  # type: (str, Any) -> Any
+        default_value = self.default_values.get(name)
+        # best effort conversion to boolean
+        if isinstance(default_value, bool):
+            value = value.lower().strip() in ("1", "true", "yes", "y")
+        elif isinstance(default_value, (int, float)):
+            try:
+                value = float(value)
+            except ValueError:
+                LOGGER.error(
+                    "Invalid config value for %s, using default value",
+                    name.lower()
+                )
+                value = default_value
+        return value
+
     def _file_path_from_env(self):
         """ Return file path if os environement was set and file exists
         """
@@ -173,7 +207,7 @@ class Config(object):
         if isfile("sqreen.ini"):
             return os.path.join(os.getcwd(), "sqreen.ini")
 
-    def __getitem__(self, name):
+    def __getitem__(self, name):  # type: (str) -> Any
         return self.config[name]
 
 
