@@ -4,8 +4,6 @@
 #
 #     https://www.sqreen.io/terms.html
 #
-import uuid
-
 from ....constants import ACTIONS
 from ....rules import RuleCallback
 from ....workflow import Transport
@@ -19,26 +17,19 @@ class HttpClientTransportCallback(RuleCallback):
         except Exception:
             host_ip, host_port = None, None
 
-        tracing_identifier_prefix = self.runner.settings.tracing_identifier_prefix
-        if tracing_identifier_prefix is not None:
-            request_identifier = uuid.uuid4()
-            tracing_identifier = "{}.{}".format(tracing_identifier_prefix, request_identifier)
-        else:
-            tracing_identifier = None
-            request_identifier = None
-
         transport = Transport({
+            "type": "http",
             "host": instance.host or host_ip,
             "host_ip": host_ip,
             "host_port": host_port,
-            "tracing_identifier_prefix": tracing_identifier_prefix,
-            "tracing_identifier": tracing_identifier,
-            "request_identifier": request_identifier,
         })
 
         context = self.runner.interface_manager.call("context")
-        if not self.runner.interface_manager.call("new_data_out_pre", context, transport) \
-                or tracing_identifier is None:
+        if not self.runner.interface_manager.call("new_data_out_pre", context, transport):
+            return
+
+        tracing_identifier = transport.get("tracing_identifier")
+        if tracing_identifier is None:
             return
 
         new_args = list(args)
