@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -7,7 +7,6 @@
 """ Low-level HTTP interaction with sqreen API
 """
 import codecs
-import collections
 import json
 import os
 import sys
@@ -15,15 +14,23 @@ from datetime import datetime
 from json import JSONEncoder
 
 from .__about__ import __version__
-from ._vendors.urllib3 import PoolManager, ProxyManager, Retry, make_headers
-from .exceptions import (InvalidJsonResponse, InvalidResponseContentType,
-                         InvalidStatusCodeResponse, StatusFailedResponse)
-from .utils import is_string, is_unicode
+from ._vendors.urllib3 import (  # type: ignore
+    PoolManager,
+    ProxyManager,
+    Retry,
+    make_headers,
+)
+from .exceptions import (
+    InvalidJsonResponse,
+    InvalidResponseContentType,
+    InvalidStatusCodeResponse,
+    StatusFailedResponse,
+)
+from .utils import Iterable, Mapping, is_string, is_unicode
 
-try:
-    # Python 2
+if sys.version_info[0] < 3:
     import urlparse
-except ImportError:
+else:
     import urllib.parse as urlparse
 
 
@@ -54,7 +61,7 @@ class SqreenJSONEncoder(JSONEncoder):
 
         # Process bytes for python3
         if IS_PYTHON_3 and isinstance(obj, bytes):
-            return obj.decode('utf-8', errors='ascii_to_hex')
+            return obj.decode("utf-8", errors="ascii_to_hex")
 
         # For unknows types, JSONEncoder will raise a TypeError, instead,
         # returns obj repr
@@ -80,19 +87,19 @@ def ascii_to_hex(exception):
         for character in invalid_part:
             # Python 2 strings
             if isinstance(character, str):
-                result.append(u'\\x{}'.format(character.encode('hex')))
+                result.append(u"\\x{}".format(character.encode("hex")))
             # Python 3 int
             elif isinstance(character, int):
-                result.append(u'\\{}'.format(hex(character)[1:]))
+                result.append(u"\\{}".format(hex(character)[1:]))
             else:
                 raise exception
-        result = (''.join(result), end)
+        result = ("".join(result), end)
         return result
 
     raise exception
 
 
-codecs.register_error('ascii_to_hex', ascii_to_hex)
+codecs.register_error("ascii_to_hex", ascii_to_hex)
 
 
 def encode_payload(payload):
@@ -100,10 +107,14 @@ def encode_payload(payload):
     payload and retry to encode it into JSON.
     """
     try:
-        return json.dumps(payload, separators=(',', ':'), cls=SqreenJSONEncoder)
+        return json.dumps(
+            payload, separators=(",", ":"), cls=SqreenJSONEncoder
+        )
     except UnicodeDecodeError:
         reencoded_payload = reencode_payload(payload)
-        return json.dumps(reencoded_payload, separators=(',', ':'), cls=SqreenJSONEncoder)
+        return json.dumps(
+            reencoded_payload, separators=(",", ":"), cls=SqreenJSONEncoder
+        )
 
 
 def reencode_payload(payload):
@@ -111,9 +122,12 @@ def reencode_payload(payload):
     """
     if is_string(payload):
         return _reencode_string(payload)
-    elif isinstance(payload, collections.Mapping):
-        return {_reencode_string(key): reencode_payload(value) for key, value in payload.items()}
-    elif isinstance(payload, collections.Iterable):
+    elif isinstance(payload, Mapping):
+        return {
+            _reencode_string(key): reencode_payload(value)
+            for key, value in payload.items()
+        }
+    elif isinstance(payload, Iterable):
         return [reencode_payload(item) for item in payload]
     else:
         return payload
@@ -123,7 +137,7 @@ def _reencode_string(string):
     """ Ensure that the string is encodable into JSON
     """
     if not is_unicode(string):
-        return string.decode('utf-8', errors='ascii_to_hex')
+        return string.decode("utf-8", errors="ascii_to_hex")
 
     return string
 
@@ -132,10 +146,10 @@ def where_crt():
     """ Returns the path of the bundled crt
     """
     current_file = os.path.split(__file__)[0]
-    return os.path.join(current_file, 'ca.crt')
+    return os.path.join(current_file, "ca.crt")
 
 
-USER_AGENT = 'sqreen-python/{}'.format(__version__)
+USER_AGENT = "sqreen-python/{}".format(__version__)
 
 
 class Urllib3Connection(object):
@@ -143,45 +157,78 @@ class Urllib3Connection(object):
 
     handle connection pooling, retry and inbound/outbound formatting
     """
-    RETRY_CONNECT_SECONDS = 10
-    RETRY_REQUEST_SECONDS = 10
-
-    MAX_DELAY = 60 * 30
 
     RETRY_STATUS = {500, 502, 503, 504, 408}
-    RETRY = Retry(total=5, method_whitelist=False,
-                  status_forcelist=RETRY_STATUS, backoff_factor=1)
-    RETRY_LONG = Retry(total=128, method_whitelist=False,
-                       status_forcelist=RETRY_STATUS, backoff_factor=1)
-    RETRY_ONCE = Retry(total=1, method_whitelist=False,
-                       status_forcelist=RETRY_STATUS, backoff_factor=1)
+    # Beware that the retry policy will block the Sqreen thread until
+    # the request is successful.
+    RETRY = Retry(
+        total=5,
+        method_whitelist=False,
+        status_forcelist=RETRY_STATUS,
+        backoff_factor=1,
+    )
+    RETRY_LONG = Retry(
+        total=15,
+        method_whitelist=False,
+        status_forcelist=RETRY_STATUS,
+        backoff_factor=1,
+    )
+    RETRY_ONCE = Retry(
+        total=1,
+        method_whitelist=False,
+        status_forcelist=RETRY_STATUS,
+        backoff_factor=1,
+    )
 
     PATH_PREFIX = "/sqreen/"
 
-    def __init__(self, server_url, proxy_url=None, user_agent=USER_AGENT):
+    def __init__(self, server_url, proxy_url=None, user_agent=USER_AGENT,
+                 use_legacy_url=False):
         self.server_url = server_url
         self.proxy_url = proxy_url
         self.parsed_server_url = urlparse.urlparse(server_url)
+        self.user_agent = user_agent
+        self.use_legacy_url = use_legacy_url
 
+        # Disable certificate pinning when using the newer domain
+        ca_certs = where_crt() if use_legacy_url else None
         if self.proxy_url:
             self.connection = ProxyManager(
                 proxy_url,
-                cert_reqs='CERT_REQUIRED',
-                ca_certs=where_crt(),
+                cert_reqs="CERT_REQUIRED",
+                ca_certs=ca_certs,
+                timeout=60,
             )
         else:
-            self.connection = PoolManager(cert_reqs='CERT_REQUIRED',
-                                          ca_certs=where_crt())
+            self.connection = PoolManager(
+                cert_reqs="CERT_REQUIRED", ca_certs=ca_certs, timeout=60
+            )
 
-        self.base_headers = make_headers(keep_alive=True, accept_encoding=True,
-                                         user_agent=user_agent)
+    def get_user_agent(self):
+        return getattr(self, "_user_agent", USER_AGENT)
+
+    def set_user_agent(self, value):
+        self.base_headers = make_headers(
+            keep_alive=True, accept_encoding=True, user_agent=value
+        )
+        self._user_agent = value
+
+    user_agent = property(get_user_agent, set_user_agent)
 
     def _url(self, url):
         """ Format and return an url based on instance path prefix
         """
         path = urlparse.urljoin(self.PATH_PREFIX, url)
-        return urlparse.urlunparse((self.parsed_server_url[
-            0], self.parsed_server_url[1], path, None, None, None))
+        return urlparse.urlunparse(
+            (
+                self.parsed_server_url[0],
+                self.parsed_server_url[1],
+                path,
+                None,
+                None,
+                None,
+            )
+        )
 
     def post(self, endpoint, data=None, headers=None, retries=None):
         """ Post a request to the backend
@@ -193,16 +240,14 @@ class Urllib3Connection(object):
 
         if data is not None:
             data = encode_payload(data)
-            headers['Content-Type'] = 'application/json'
+            headers["Content-Type"] = "application/json"
 
         # Add base headers
         headers.update(self.base_headers)
 
-        response = self.connection.urlopen('POST',
-                                           url,
-                                           headers=headers,
-                                           body=data,
-                                           retries=retries)
+        response = self.connection.urlopen(
+            "POST", url, headers=headers, body=data, retries=retries
+        )
 
         return self._parse_response(response)
 
@@ -217,28 +262,29 @@ class Urllib3Connection(object):
         # Add base headers
         headers.update(self.base_headers)
 
-        response = self.connection.urlopen('GET',
-                                           url,
-                                           headers=headers,
-                                           retries=retries)
+        response = self.connection.urlopen(
+            "GET", url, headers=headers, retries=retries
+        )
 
         return self._parse_response(response)
 
     def _parse_response(self, response):
         """ Try to decode response body to json
         """
-        if response.status < 200 or response.status > 300:
+        if response.status < 200 or response.status >= 300:
             raise InvalidStatusCodeResponse(response.status, response.data)
 
-        if response.headers.get('Content-Type') != 'application/json':
-            raise InvalidResponseContentType(response.headers.get('Content-Type'))
+        if response.headers.get("Content-Type") != "application/json":
+            raise InvalidResponseContentType(
+                response.headers.get("Content-Type")
+            )
 
         try:
-            json_response = json.loads(response.data.decode('utf-8'))
+            json_response = json.loads(response.data.decode("utf-8"))
         except ValueError as exc:
             raise InvalidJsonResponse(exc)
 
-        if json_response.get('status', False) is False:
+        if json_response.get("status", False) is False:
             raise StatusFailedResponse(json_response)
 
         return json_response
