@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,18 +8,16 @@
 """
 import logging
 
-from .dbapi2 import CustomConnection, CustomCursor
+from .dbapi2 import CustomConnectionBase, CustomCursorBase
 from .import_hook import ImportHookStrategy
 
 LOGGER = logging.getLogger(__name__)
 
 
 def wrap_custom_register_type(original):
-
     def custom_register_type(obj, scope=None):
-        if isinstance(scope, (CustomConnection, CustomCursor)):
-            scope = scope._obj
-
+        if isinstance(scope, (CustomConnectionBase, CustomCursorBase)):
+            scope = scope.__wrapped__
         return original(obj, scope)
 
     return custom_register_type
