@@ -382,14 +382,13 @@ class MetricsStore(object):
             # Retrieve the current period
             # Call the finalize method on the aggregator
 
-            finished = {}
+            finished = dict(metric)
+            finished["name"] = name
             finished["observation"] = aggregator.finalize(
                 metric["observation"]
             )
             usual_finish = metric["start"] + timedelta(seconds=metric["period"])
             finished["finish"] = at if at < usual_finish else usual_finish
-            finished["name"] = name
-            finished["start"] = metric["start"]
             self.store.append(finished)
 
         # Reset the period
