@@ -9,6 +9,8 @@
 import sys
 import traceback
 
+from .utils import now
+
 
 def traceback_formatter(backtrace):
     """ Accept a backtrace in the format of traceback.extract_tb or
@@ -41,6 +43,19 @@ def raw_traceback_formatter(raw_backtrace):
     return traceback_formatter(traceback.extract_tb(raw_backtrace))
 
 
+def backtrace_to_stack_trace(backtrace):
+    """Adapt backtraces to the signal stack trace format.
+    """
+    for frame in backtrace:
+        abs_path = frame.get("file")
+        yield {
+            "function": frame.get("method", frame.get("function")),
+            "abs_path": abs_path,
+            "lineno": frame.get("line_number", frame.get("line")),
+            "in_app": "sqreen" not in abs_path if abs_path else False,
+        }
+
+
 class RemoteException(object):
     def __init__(
         self,
@@ -51,9 +66,15 @@ class RemoteException(object):
         exception_payload=None,
         request_payload=None,
         stack=None,
+        at=None,
     ):
         self.exception_class = name
         self.exception_msg = msg
+
+        if at is None:
+            at = now()
+        self.at = at
+
         self.raw_backtrace = raw_backtrace
         if raw_backtrace is not None:
             self.backtrace = raw_traceback_formatter(self.raw_backtrace)
@@ -115,3 +136,43 @@ class RemoteException(object):
         )
 
         return base_payload
+
+    def to_signal(self):
+        """ Exception signal.
+        """
+        signal = dict(
+            type="point",
+            signal_name="sq.agent.exception",
+            payload_schema="sqreen_exception/2020-01-01T00:00:00.000Z",
+        )
+
+        rule_name = self.callback_payload.pop("rule_name", None)
+        if rule_name is not None:
+            rulespack_id = self.callback_payload.pop(
+                "rulespack_id", "0000000000000000000000000000000000000000")
+            signal["source"] = "sqreen:rule:{}:{}".format(rulespack_id, rule_name)
+        else:
+            signal["source"] = "sqreen:agent:python"
+
+        signal["time"] = self.at
+        signal["location"] = {
+            "stack_trace": list(backtrace_to_stack_trace(self.backtrace))
+        }
+
+        infos = {}
+        if self.callback_payload:
+            infos["callback"] = dict(self.callback_payload)
+
+        if self.exception_payload:
+            infos["exception"] = self.exception_payload
+
+        signal["payload"] = {
+            "klass": self.exception_class,
+            "message": self.exception_msg,
+            "infos": infos,
+        }
+
+        if self.request_payload:
+            signal["payload"].update(self.request_payload)
+
+        return signal
