@@ -88,7 +88,6 @@ def check_condition_wrapper(wrapped, callback, condition, lifecycle):
             )
             return None
 
-        LOGGER.debug("Running %r on callback %r", lifecycle, callback)
         # Execute the hook otherwise with the original args
         return wrapped(inst, args, kwargs, **options)
 
@@ -142,6 +141,17 @@ def perf_monitoring_wrapper(wrapped, storage, name):
     return wrapper
 
 
+def debug_log_wrapper(wrapped, lifecycle, callback):
+
+    @wraps(wrapped)
+    def wrapper(*args, **kwargs):
+        LOGGER.debug("Running %r on callback %r", lifecycle, callback)
+        return wrapped(*args, **kwargs)
+
+    wrapper.__wrapped__ = wrapped
+    return wrapper
+
+
 class RuleCallback(BaseCallback):
 
     SECTIONS = ("request", "response", "params", "headers", "context")
@@ -198,6 +208,7 @@ class RuleCallback(BaseCallback):
         self.call_count_interval = call_count_interval
         self.call_counts = {"pre": 0, "post": 0, "failing": 0}
 
+        self._apply_debug_log()
         self._apply_conditions()
         self._apply_call_count()
         self._apply_perf_monitoring()
@@ -273,6 +284,16 @@ class RuleCallback(BaseCallback):
                     lifecycle_method, self, conditions, lifecycle)
                 setattr(self, lifecycle, wrapped)
 
+    def _apply_debug_log(self):
+        if not LOGGER.isEnabledFor(logging.DEBUG):
+            return
+
+        for lifecycle in LIFECYCLE_METHODS.values():
+            lifecycle_method = getattr(self, lifecycle, None)
+            if lifecycle_method is not None:
+                wrapped = debug_log_wrapper(lifecycle_method, lifecycle, self)
+                setattr(self, lifecycle, wrapped)
+
     def _apply_call_count(self):
         # Only count calls if call_count_interval is > 0
         if self.call_count_interval == 0:
