@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -9,18 +9,17 @@
 
 from logging import getLogger
 
-from ..runtime_storage import runtime
 from .matcher_callback import MatcherRule
 
 LOGGER = getLogger(__name__)
 
 
 class CrawlerUserAgentMatchesMetricsCB(MatcherRule):
-    def pre(self, original, *args, **kwargs):
+    def pre(self, instance, args, kwargs, **options):
         """ For each request, record an observation with the user_agent without
         case modification.
         """
-        request = runtime.get_current_request()
+        request = self.storage.get_current_request()
 
         if not request:
             LOGGER.warning("No request was recorded abort")
@@ -32,4 +31,4 @@ class CrawlerUserAgentMatchesMetricsCB(MatcherRule):
             return
 
         if self.match(user_agent):
-            self.record_observation('crawler', user_agent, 1)
+            self.record_observation("crawler", user_agent, 1)
