@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
 #
 """ Base class for matcher based rules
 """
-from collections import Mapping
 from logging import getLogger
 
 from ..exceptions import InvalidArgument
 from ..matcher import Matcher
 from ..rules import RuleCallback
+from ..utils import Mapping
 
 LOGGER = getLogger(__name__)
 
@@ -30,7 +30,7 @@ class MatcherRule(RuleCallback):
             raise InvalidArgument(msg.format(type(self.data)))
 
         try:
-            raw_patterns = self.data['values']
+            raw_patterns = self.data["values"]
         except KeyError:
             msg = "No key 'values' in data (had {})"
             raise InvalidArgument(msg.format(self.data.keys()))
