@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,7 +8,6 @@
 """
 from logging import getLogger
 
-from ..runtime_storage import runtime
 from .regexp_rule import RegexpRule
 
 LOGGER = getLogger(__name__)
@@ -16,9 +15,9 @@ LOGGER = getLogger(__name__)
 
 class UserAgentMatchesCBFramework(RegexpRule):
 
-    def pre(self, original, *args, **kwargs):
+    def pre(self, instance, args, kwargs, **options):
 
-        request = runtime.get_current_request()
+        request = self.storage.get_current_request()
 
         if not request:
             LOGGER.warning("No request was recorded abort")
@@ -32,11 +31,10 @@ class UserAgentMatchesCBFramework(RegexpRule):
         if not match:
             return
 
-        infos = {'found': match}
+        infos = {"found": match, "in": user_agent}
         self.record_attack(infos)
 
-        return {'status': 'raise', 'data': match,
-                'rule_name': self.rule_name}
+        return {"status": "raise", "data": match, "rule_name": self.rule_name}
 
 
 UserAgentMatchesCBDjango = UserAgentMatchesCBFramework
