@@ -19,19 +19,23 @@ from logging import getLogger
 from threading import Event, Thread
 
 from .__about__ import __version__
+from ._vendors.urllib3 import PoolManager, ProxyManager
 from ._vendors.urllib3.exceptions import HTTPError  # type: ignore
+from ._vendors.urllib3.util import Timeout
+from .actions import ActionStore
 from .config import CONFIG
 from .constants import CHANGELOG_URL, COMPATIBILITY_URL, STATUS_URL
 from .deliverer import get_deliverer
+from .ecosystem import init as init_ecosystem
 from .exceptions import (
     InvalidApplicationName,
-    InvalidSignature,
     UnsupportedFrameworkVersion,
     UnsupportedPythonVersion,
 )
 from .frameworks import debug
 from .http_client import Urllib3Connection
 from .instrumentation import Instrumentation
+from .interface_manager import InterfaceManager
 from .log import configure_root_logger
 from .metrics import MetricsStore
 from .remote_command import RemoteCommand
@@ -51,29 +55,29 @@ from .runtime_infos import (
     get_process_cmdline,
 )
 from .session import InvalidSession, InvalidToken, Session
+from .signal_client import get_signal_client
 from .utils import (
+    HAS_TYPING,
     configure_newrelics_ignore_exception,
     configure_raven_breadcrumbs,
 )
 
+if sys.version_info[0] < 3:
+    import urlparse
+
+    from ._vendors.concurrent.futures import ThreadPoolExecutor, TimeoutError
+else:
+    import urllib.parse as urlparse
+
+    from concurrent.futures import ThreadPoolExecutor, TimeoutError
+
+if HAS_TYPING:
+    from typing import Sequence, Optional
+
 RUNNER_THREAD = None
-RUNNER_THREAD_PID = None
 RUNNER_THREAD_RESTARTING = False
 RUNNER_LOCK = threading.RLock()
 
-# Instrumentation timeout, max time waiting for the runner thread to instrument the application
-try:
-    INSTRUMENTATION_TIMEOUT = int(os.getenv("SQREEN_INSTRUMENTATION_TIMEOUT", 6))
-except ValueError:
-    INSTRUMENTATION_TIMEOUT = 6
-
-# Graceful timeout, max time waiting for the runner thread to exit before exiting
-try:
-    GRACEFUL_TIMEOUT = int(os.getenv("SQREEN_GRACEFUL_TIMEOUT", 5))
-except ValueError:
-    GRACEFUL_TIMEOUT = 5
-GRACEFUL_FAIL_MSG = "Sqreen thread didn't exit after %s seconds"
-
 LOGGER = getLogger(__name__)
 
 
@@ -94,20 +98,13 @@ def should_start_runner_thread():
     should be restarted. Beware, this function *must not* call any hooks.
     """
     global RUNNER_THREAD
-    global RUNNER_THREAD_PID
 
-    runner_stopped = False
-    if RUNNER_THREAD is not None \
-            and RUNNER_THREAD.runner is not None \
-            and RUNNER_THREAD.runner.stop is True:
-        runner_stopped = True
-
-    return RUNNER_THREAD_PID != os.getpid() and not runner_stopped
+    return getattr(RUNNER_THREAD, "pid", None) != os.getpid() and not \
+        getattr(getattr(RUNNER_THREAD, "runner", None), "stop", False)
 
 
 def before_hook_point():
     global RUNNER_THREAD
-    global RUNNER_THREAD_PID
     global RUNNER_THREAD_RESTARTING
 
     # /!\ The order of execution in this function must not change
@@ -127,14 +124,12 @@ def before_hook_point():
             # Else, restart the thread
             try:
                 RUNNER_THREAD_RESTARTING = True
-                runner = RUNNER_THREAD.replace()
-                runner.start()
-
-                RUNNER_THREAD = runner
-                RUNNER_THREAD_PID = os.getpid()
+                RUNNER_THREAD = RUNNER_THREAD.replace()
+                RUNNER_THREAD.start()
 
                 LOGGER.info(
-                    "Sucessfully started runner thread for pid: %d", os.getpid()
+                    "Successfully started runner thread for pid: %d",
+                    RUNNER_THREAD.pid
                 )
             finally:
                 RUNNER_THREAD_RESTARTING = False
@@ -174,23 +169,25 @@ def get_app_name():
     return app_name
 
 
-def get_session(url, token, app_name=None, proxy_url=None):
+def get_session(url, token, app_name=None, proxy_url=None, use_legacy_url=False):
     """ Create a connection with the endpoint URL and a session. Returns only
     the session.
     """
     LOGGER.warning(
-        "Connection to %s%s",
+        "Connection to %s%s%s",
+        "legacy " if use_legacy_url else "",
         url,
         " using proxy %s" % proxy_url if proxy_url else "",
     )
-    con = Urllib3Connection(url, proxy_url)
+    con = Urllib3Connection(url, proxy_url=proxy_url, use_legacy_url=use_legacy_url)
     session = Session(con, token, app_name)
     return session
 
 
 def post_app_sqreen_exception(data):
     """Post a Sqreen exception happening at application level.
-    This function does not need a session.
+    This function does not need a session. It always uses the legacy backend URL
+    until all customers are migrated.
 
     This function is used to report instrumentation errors. It should never
     fail.
@@ -202,7 +199,8 @@ def post_app_sqreen_exception(data):
     try:
         app_name = get_app_name()
         session = get_session(
-            CONFIG["URL"], token, app_name, proxy_url=CONFIG["PROXY_URL"]
+            CONFIG["LEGACY_URL"], token, app_name, proxy_url=CONFIG["PROXY_URL"],
+            use_legacy_url=True
         )
         session.post_app_sqreen_exception(data)
     except Exception:
@@ -213,15 +211,14 @@ def start():
     # type: () -> None
     """ Start the background thread and start protection
     """
-    # Check if the agent is not disabled first
-    try:
-        disabled = int(os.getenv("SQREEN_DISABLE", 0))
-    except ValueError:
-        disabled = 0
-
     # Configure logging
     configure_root_logger(CONFIG["LOG_LEVEL"], CONFIG["LOG_LOCATION"])
 
+    # Check if the agent is not disabled
+    if CONFIG["DISABLE"]:
+        LOGGER.debug("Sqreen agent is disabled.")
+        return
+
     # Retrieve the command used to launch the process
     command = get_process_cmdline()
 
@@ -247,10 +244,6 @@ def start():
         LOGGER.debug("Sqreen agent is disabled when running tests.")
         return
 
-    if disabled:
-        LOGGER.debug("Sqreen agent is disabled.")
-        return
-
     try:
         runtime_infos = RuntimeInfos().all()
     except UnsupportedFrameworkVersion as exception:
@@ -297,33 +290,20 @@ def start():
     configure_newrelics_ignore_exception()
 
     global RUNNER_THREAD
-    global RUNNER_THREAD_PID
 
     # Check for double instrumentation
     if RUNNER_THREAD is None:
         try:
-            runner = RunnerThread(runtime_infos)
-            runner.start()
+            RUNNER_THREAD = RunnerThread(runtime_infos)
+            RUNNER_THREAD.start()
 
-            done = runner.unblock_main_thread.wait(INSTRUMENTATION_TIMEOUT)
+            done = RUNNER_THREAD.unblock_main_thread.wait(CONFIG["INSTRUMENTATION_TIMEOUT"])
             if done is not True:
-                post_app_sqreen_exception(RemoteException(
-                    "InstrumentationTimeout",
-                    "Timeout during instrumentation",
-                    exception_payload={
-                        "runner_thread_started": runner.started,
-                        "runner_thread_was_alive": runner.is_alive(),
-                        "instrumentation_enabled": runner.instrumentation.enabled,
-                        "runtime_infos": runtime_infos,
-                    },
-                    stack=runner.stacktrace,
-                ).to_dict())
-
                 LOGGER.critical(
                     "Sqreen could not instrument your application in less than %ds.\n"
                     "Please check your account is not expired and your application has network connectivity.\n"
                     "You can also check Sqreen status on %s.",
-                    INSTRUMENTATION_TIMEOUT, STATUS_URL)
+                    CONFIG["INSTRUMENTATION_TIMEOUT"], STATUS_URL)
 
         except Exception:
             LOGGER.critical(
@@ -332,8 +312,29 @@ def start():
             )
             return
 
-        RUNNER_THREAD = runner
-        RUNNER_THREAD_PID = os.getpid()
+
+def sqreen_compatible_backends(base_urls, proxy_url=None):
+    # type: (Sequence[str], Optional[str]) -> bool
+    """ Detect Sqreen backend reachability.
+    """
+    LOGGER.debug("Detecting reachability to: %s", ", ".join(base_urls))
+    timeout_policy = Timeout(total=30)
+    http = PoolManager(timeout=timeout_policy) if proxy_url is None \
+        else ProxyManager(proxy_url=proxy_url, timeout=timeout_policy)
+
+    def request(base_url):
+        url = urlparse.urljoin(base_url, "/ping")
+        r = http.urlopen("GET", url)
+        return r.status == 200
+
+    try:
+        with ThreadPoolExecutor(max_workers=2) as executor:
+            return all(executor.map(request, base_urls, timeout=30))
+    except TimeoutError:
+        return False
+    except Exception:
+        LOGGER.debug("Couldn't detect domains", exc_info=True)
+        return False
 
 
 def get_initial_features(config_features, login_features):
@@ -372,18 +373,29 @@ def get_runner(
     observations_queue,
     instrumentation,
     settings,
+    interface_manager,
 ):
     initial_features = get_initial_features(
         config["INITIAL_FEATURES"], login_result.get("features", {})
     )
 
+    # Get a signal client if the feature is enabled
+    use_signals = initial_features.get("use_signals", False)
+    signal_client = get_signal_client(
+        session=session,
+        batch_size=initial_features.get("batch_size", 0),
+        max_staleness=initial_features.get("max_staleness", 0)
+    ) if use_signals else None
+
     # Get the right deliverer according to initial features
     deliverer = get_deliverer(
         initial_features.get("batch_size", 0),
         initial_features.get("max_staleness", 0),
         session,
+        signal_client=signal_client,
     )
     remote_command = RemoteCommand.with_production_commands()
+    action_store = ActionStore()
     metrics_store = MetricsStore()
     metrics_store.register_production_aggregators()
     metrics_store.register_default_metrics()
@@ -397,9 +409,12 @@ def get_runner(
         remote_command,
         runtime_infos,
         instrumentation,
+        action_store,
         metrics_store,
         settings,
         initial_features,
+        signal_client=signal_client,
+        interface_manager=interface_manager,
     )
     return runner
 
@@ -411,7 +426,7 @@ class RunnerThread(Thread):
     name = "SqreenRunnerThread"
 
     def __init__(self, runtime_infos, queue=None, observations_queue=None,
-                 instrumentation=None, settings=None):
+                 instrumentation=None, settings=None, interface_manager=None):
         super(RunnerThread, self).__init__()
         self.daemon = True
         self.queue = queue or CappedQueue(MAX_QUEUE_LENGTH)
@@ -422,8 +437,12 @@ class RunnerThread(Thread):
             self.observations_queue, self.queue, before_hook_point
         )
         self.settings = settings or RunnerSettings()
+        if CONFIG["BETA_STATIC_INSTRUMENTATION"]:
+            interface_manager = interface_manager or InterfaceManager()
+        self.interface_manager = interface_manager
         self.runner = None
         self.started = False
+        self.pid = None
 
     @property
     def stacktrace(self):
@@ -438,7 +457,6 @@ class RunnerThread(Thread):
         """
         self.started = True
         LOGGER.debug("Starting Sqreen %s", __version__)
-        # FIXME this part should be more tested
         while True:
             session = None
             self.runner = None
@@ -471,9 +489,20 @@ class RunnerThread(Thread):
 
                 LOGGER.warning("Using token %r and application name %r", token, app_name)
 
-                # Initiate HTTP connection to the backend and the session
+                if not CONFIG["NO_SNIFF_DOMAINS"]:
+                    # Detect if the configured URLs are reachable.
+                    use_legacy_url = not sqreen_compatible_backends(
+                        [CONFIG["URL"], CONFIG["INGESTION_URL"]], proxy_url=CONFIG["PROXY_URL"])
+                else:
+                    # Force use of the configured URLs.
+                    use_legacy_url = False
+
+                # Initiate HTTP connection to the backend and the session, fallback to the legacy URL
+                # if the new URLs are not available.
+                session_url = CONFIG["LEGACY_URL"] if use_legacy_url else CONFIG["URL"]
                 session = get_session(
-                    CONFIG["URL"], token, app_name, proxy_url=CONFIG["PROXY_URL"]
+                    session_url, token, app_name, proxy_url=CONFIG["PROXY_URL"],
+                    use_legacy_url=use_legacy_url
                 )
 
                 runtime_infos = dict(self.runtime_infos)
@@ -511,6 +540,10 @@ class RunnerThread(Thread):
 
                 LOGGER.info("Login success")
 
+                if self.interface_manager is not None:
+                    LOGGER.debug("Initializing ecosystem")
+                    init_ecosystem(self.interface_manager)
+
                 self.runner = get_runner(
                     CONFIG,
                     login_result,
@@ -520,6 +553,7 @@ class RunnerThread(Thread):
                     self.observations_queue,
                     self.instrumentation,
                     self.settings,
+                    self.interface_manager,
                 )
 
                 # Process the initial commands
@@ -555,12 +589,15 @@ class RunnerThread(Thread):
                 self._random_sleep()
             except Exception as exc:
                 LOGGER.exception("An unexpected exception occurred")
-                if isinstance(exc, InvalidSignature):
-                    remote_exception = RemoteException.from_exc_info(
-                        exc.exc_info, callback_payload=exc.callback_payload  # pylint: disable=no-member
-                    )
-                else:
-                    remote_exception = RemoteException.from_exc_info()
+
+                callback_payload = getattr(exc, "callback_payload", None)
+                exception_payload = getattr(exc, "exception_payload", None)
+                request_payload = getattr(exc, "request_payload", None)
+                remote_exception = RemoteException.from_exc_info(
+                    callback_payload=callback_payload,
+                    exception_payload=exception_payload,
+                    request_payload=request_payload,
+                )
 
                 if session is not None and session.is_connected():
                     try:
@@ -599,6 +636,7 @@ class RunnerThread(Thread):
 
     def start(self):
         """Start the runner thread."""
+        self.pid = os.getpid()
         atexit.register(self.graceful_stop)
         return super(RunnerThread, self).start()
 
@@ -610,15 +648,16 @@ class RunnerThread(Thread):
             LOGGER.warning("Quick exit, don't logout")
             return
 
+        GRACEFUL_FAIL_MSG = "Sqreen thread didn't exit after %s seconds"
         try:
-            self.join(GRACEFUL_TIMEOUT)
+            self.join(CONFIG["GRACEFUL_TIMEOUT"])
         except RuntimeError:
             logger = getLogger(__name__)
-            logger.warning(GRACEFUL_FAIL_MSG, GRACEFUL_TIMEOUT)
+            logger.warning(GRACEFUL_FAIL_MSG, CONFIG["GRACEFUL_TIMEOUT"])
         else:
             if self.isAlive():
                 logger = getLogger(__name__)
-                logger.warning(GRACEFUL_FAIL_MSG, GRACEFUL_TIMEOUT)
+                logger.warning(GRACEFUL_FAIL_MSG, CONFIG["GRACEFUL_TIMEOUT"])
 
     def replace(self, **kwargs):
         """Create a new runner thread instance."""
@@ -627,6 +666,7 @@ class RunnerThread(Thread):
             observations_queue=self.observations_queue,
             instrumentation=self.instrumentation,
             settings=self.settings,
+            interface_manager=self.interface_manager,
         )
         nkwargs.update(kwargs)
         return self.__class__(RuntimeInfos().all(), **nkwargs)
